/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kühntopf <andreas@kuehntopf.org>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic;

import java.io.File;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.filechooser.FileFilter;
import org.jvnet.substance.SubstanceDefaultLookAndFeel;
import org.jvnet.substance.button.ClassicButtonShaper;
import org.jvnet.substance.skin.OfficeSilver2007Skin;
import org.jvnet.substance.theme.SubstanceOrangeTheme;
import org.kuehntopf.streamtastic.core.storage.Config;
import org.kuehntopf.streamtastic.core.streamripper.ProgramVersionStatus;
import org.kuehntopf.streamtastic.core.streamripper.StreamRipper;
import org.kuehntopf.streamtastic.gui.SelectDestinationFrame;
import org.kuehntopf.streamtastic.gui.StreamtasticFrame;
import org.kuehntopf.streamtastic.utils.Utilities;

/**
 * Main application class
 * @author akuehntopf
 */
public class StreamtasticApp 
{
    /**
     * Returns the application's Version
     * @return version
     */
    public static String 
    getVersion()
    {
        return "0.3.7";
    }
    
    /**
     * Checks if Streamripper is available.
     * If it isn't the App is terminated
     */
    private static void
    checkStreamripperAvailable()
    {
        // If Config override is set, use that
        String streamripperCmd = Config.getInstance().getProperty("STREAMRIPPER_CMD");
        if (streamripperCmd != null)
        {
            StreamRipper.STREAMRIPPER_CMD = streamripperCmd;
        }

        // Check if its available at either $path or at the location specified in the
        // Config file
        ProgramVersionStatus srStatus = StreamRipper.getVersionStatus();

        switch (srStatus)
        {
            case NOTAVAILABLE:
                // Ask if the User wants to select it manually
                int ret = JOptionPane.showConfirmDialog(null, 
                        Utilities.getString("STREAMRIPPER_NOT_INSTALLED"),
                        Utilities.getString("ERROR"),
                        JOptionPane.ERROR_MESSAGE,
                        JOptionPane.YES_NO_OPTION);

                switch (ret)
                {
                    case JOptionPane.YES_OPTION:
                        JFileChooser chooser = new JFileChooser();
                        chooser.setFileFilter(new FileFilter() {

                        @Override
                        public boolean accept(File arg0) {
                            if (arg0.isDirectory() || (arg0.getName().contains("streamripper")) && arg0.canExecute())
                                return true;
                            else
                                return false;
                        }

                        @Override
                        public String getDescription() {
                            return "Streamripper Program";
                        }
                        });
                        
                        chooser.showOpenDialog(null);

                        // Check if "Cancel" was pressed
                        if (chooser.getSelectedFile() == null)
                        {
                            System.exit(1);
                        }
                        else
                        {
                            // Save new Streamripper Command
                            Config.getInstance().setProperty("STREAMRIPPER_CMD",
                                   chooser.getSelectedFile().getPath());
                            Config.getInstance().saveProperties();

                            // Redo checks (should also check version)
                            checkStreamripperAvailable();
                            return;
                        }
                        break;
                    case JOptionPane.NO_OPTION:
                        System.exit(1);
                }
                
            case NOTSUPPORTED:
                Utilities.showLocalizedError("STREAMRIPPER_VERSION_UNSUPPORTED");
                System.exit(1);
                break;
        }        
    }
    
    /**
     * Checks if the destination directory
     * has been set. Prompts the user for it if it wasn't set up yet.
     * If the user chooses to cancel the program is terminated.
     */
    private static void
    checkDestinationDirectory()
    {
        String dest = Config.getInstance().getProperty("DESTINATION_DIR");
        if (dest == null || dest.equals(""))
        {
            SelectDestinationFrame s = new SelectDestinationFrame();
            s.setModal(true);
            s.setVisible(true);
        }
    }
    
    
    /**
     * Program entry point
     * @param args the commandline arguments
     */
    public static void 
    main(String[] args)
    throws UnsupportedLookAndFeelException
    {
        // Set LAF
        JFrame.setDefaultLookAndFeelDecorated(true);
        JDialog.setDefaultLookAndFeelDecorated(true);
        UIManager.setLookAndFeel(new SubstanceDefaultLookAndFeel());
        SubstanceDefaultLookAndFeel.setCurrentTheme(new SubstanceOrangeTheme());
        SubstanceDefaultLookAndFeel.setSkin(new OfficeSilver2007Skin());
        SubstanceDefaultLookAndFeel.setCurrentButtonShaper(
                new ClassicButtonShaper());
        
        // Pre-run checks
        checkStreamripperAvailable();
        checkDestinationDirectory();
               
        // Start Frame
        SwingUtilities.invokeLater(new Runnable() 
        {
            public void run() 
            {
                new StreamtasticFrame().setVisible(true);
            }
        });
    }
}
