/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kühntopf <andreas@kuehntopf.org>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic.core;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;

import org.kuehntopf.streamtastic.core.shoutcastfeed.ShoutcastStation;
import org.kuehntopf.streamtastic.core.storage.Config;
import org.kuehntopf.streamtastic.core.streamripper.StreamRipper;
import org.kuehntopf.streamtastic.core.streamripper.StreamRipperStatus;
import org.kuehntopf.streamtastic.utils.Utilities;

/**
 * This class manages StreamRipper instances
 * and sends Messages across the bus accordingly.
 * @author akuehntopf
 */
public class RippingManager extends Observable implements Observer  
{
    ArrayList<StreamRipper>         _currentRips;
    Set<Station>                    _currentStations;
    private static RippingManager   _instance = null;   
    
    /**
     * Creates a new RippingManager instance.
     * This is private for Singleton reasons
     */
    private 
    RippingManager()
    {
        _currentRips = new ArrayList<StreamRipper>();
        _currentStations = new HashSet<Station>();
    }
    
    /**
     * Returns the singleton instance of this
     * class
     */
    public static RippingManager 
    getInstance()
    {
        if (_instance == null)
            _instance = new RippingManager();
        return _instance;
    }
    
    /**
     * Starts a new Download
     */
    public void 
    createRip(Station s)
    {
        // Get Destination dir from Config
        String destinationDir = Config.getInstance()
                                    .getProperty("DESTINATION_DIR");
        
        // Create a new Ripper and 
        // start it
        StreamRipper r = new StreamRipper(s, destinationDir);
        _currentRips.add(r);
        r.addObserver(this);
        r.start();
        s.setRecording(true);
        _currentStations.add(s);
        
        // Notify Observers of the newly added
        // download
        setChanged();
        notifyObservers(new CompoundMessage(Message.RIP_CREATED, r));        
    }         
    
    /**
     * Removes a Download
     */
    public void 
    removeRip(StreamRipper r)
    {
        // Tell Station it isn't downloaded anymore
        r.getStation().setRecording(false);
        _currentRips.remove(r);
        _currentStations.remove(r.getStation());
        setChanged();
        notifyObservers(new CompoundMessage(Message.RIP_REMOVED, r));
    }

    /**
     * Update method called by the StreamRippers
     * This means the StreamRipper has a new status available
     * @param obs Observable
     * @param obj Objects
     */
    public void 
    update(Observable obs, Object obj)
    {       
        StreamRipper rip = (StreamRipper)obs;
                
        // If no status has been set yet,
        // we cannot check anything, so just return       
        if (rip.getStatus() == null)
            return;
        
        // If terminated, check the reason
        if (rip.getStatus().equals(StreamRipperStatus.TERMINATED))
        {
            // If it got terminated by error: Show it
            if (rip.getError() != null)
            {
                Utilities.showError(
                    Utilities.getString("STREAMRIPPER_TERMINATED")+ 
                                        "\nCode: " + rip.getError() +
                                        "\nUrl: " + rip.getUrl());
            }
            
            // Has been terminated, remove it
            // and exit this method to prevent sending two messages
            removeRip(rip);
            return;
        }      
        
        // Notify observers
        setChanged();
        notifyObservers(new CompoundMessage(Message.RIP_UPDATED, rip));
    }
    
    /**
     * Checks if a Station is already being ripped
     */
    public boolean
    isRipping(Station s)
    {
        if (s instanceof ShoutcastStation)
            s = (ShoutcastStation)s;
        return _currentStations.contains(s);
    }
    
    /**
     * Returns a list of all known StreamRippers
     * @return list of streamrippers
     */
    public ArrayList<StreamRipper> 
    getRippers()
    {
        return _currentRips;
    }
}
