/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kühntopf <andreas@kuehntopf.org>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic.core.shoutcastfeed;

import java.net.URLEncoder;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.kuehntopf.streamtastic.core.storage.Config;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Shoutcast Yellow pages class
 * This downloads genres from the shoutcast web service
 * and can also get a list of stations for a specific genre.
 * @author akuehntopf
 */
public class ShoutcastFeed 
{
    private static String G_URL = "http://yp.shoutcast.com/sbin/newxml.phtml";
    private static String S_URL = G_URL + "?genre=";
    private static String F_URL = G_URL + "?search="; 
    
    /**
     * Returns a list of 
     * all registered genres from the Shoutcast
     * Yellowpages
     * @return a list of genres
     */
    public static ArrayList<String> 
    getGenres() 
    throws ShoutcastFeedException
    {
        ArrayList<String> ret = new ArrayList<String>();
        
        try
        {
            // Open URL connection and parse XML document
            Document doc = getDocForURL(G_URL);
            
            // Loop all found genre nodes and
            // append to return list
            NodeList genreNodes = doc.getElementsByTagName("genre");
            for (int s=0; s< genreNodes.getLength(); s++)
            {
                Node currentGenreNode = genreNodes.item(s);
                ret.add(getAttributeValue(currentGenreNode, "name"));
            }
                       
        } catch (Exception e)
        {
           throw new ShoutcastFeedException(e);
        }        
        return ret;
    }
    
    /**
    * Returns a list of stations for
    * the given genre
    * @param genre Genre to list
    * @return a list of stations
    */
    
    public static ArrayList<ShoutcastStation>
    getStations(String genre,String bitrate)
    throws ShoutcastFeedException
    {
        return parseStations(S_URL, genre, bitrate);
    }
    
     /**
     * Returns a list of stations for
     * the given search term
     * @param search term to list
     * @return a list of stations
     */
    
    public static ArrayList<ShoutcastStation>
    getSearchResult(String searchTerm, String bitrate)
    throws ShoutcastFeedException
    {
        return parseStations(F_URL, searchTerm, bitrate);
    }
    
    /**
     * Returns a list of stations for
     * the given genre or search term
     * @param genre Genre to list
     * @return a list of stations
     */
    
    private static ArrayList<ShoutcastStation>
    parseStations(String URL, String querry, String bitrate) 
    throws ShoutcastFeedException
    {
        String options="";
        
        // Get list length from Config
        String maxListlength = Config.getInstance().getProperty("LIST_LENGTH");
        
        // handling of options saved in config.properties should be relocated
        if (maxListlength==null)
        {
            maxListlength= "500";
            Config.getInstance().setProperty("LIST_LENGTH",maxListlength);
        }
        // if maxListlength is set to "0" the whole list will be displayed
        // this code should be relocated when preferences view is added
        if (!maxListlength.equals("0"))
            options +="&limit="+maxListlength;
        
        if (bitrate!=null)
            options+= "&br="+bitrate;

        ArrayList<ShoutcastStation> ret = new ArrayList<ShoutcastStation>();
        
        try
        {            
            Document doc = getDocForURL(URL + URLEncoder.encode(querry,"UTF-8") + options);
            // Loop all found genre nodes and
            // append to return list
            NodeList stationNodes = doc.getElementsByTagName("station");
            for (int s=0; s< stationNodes.getLength(); s++)
            {
                Node currentStationNode = stationNodes.item(s);
                ShoutcastStation sta = new ShoutcastStation();
               
                sta.setName(getAttributeValue(currentStationNode, "name"));
                sta.setMimetype(getAttributeValue(currentStationNode, "mt"));
                sta.setId(Integer.valueOf(
                        getAttributeValue(currentStationNode, "id")));
                sta.setBitrate(Integer.valueOf(
                        getAttributeValue(currentStationNode, "br")));
                sta.setGenre(getAttributeValue(currentStationNode, "genre"));
                sta.setListeners(Integer.valueOf(
                        getAttributeValue(currentStationNode, "lc")));
                sta.setCurrentTrack(getAttributeValue(currentStationNode, "ct"));
                
                ret.add(sta);
            }
            
        } catch (Exception e)
        {
            throw new ShoutcastFeedException(e);
        }        
        return ret;
    }
    
    /**
     * Returns an attribute's value
     * @param n Node
     * @param attrib Attribute name
     * @return Value
     */
    public static String 
    getAttributeValue(Node n, String attrib)
    {
        return n.getAttributes().getNamedItem(attrib).getNodeValue();
    }
    
    /**
     * Returns a parsed XML Document for 
     * an URL
     * @param url URL to parse
     * @return Document
     */
    public static Document 
    getDocForURL(String url) 
    throws Exception
    {
        Document doc = null;
        
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder = factory.newDocumentBuilder();
        doc = builder.parse(url);            
        
        return doc;
    }   
}