/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kühntopf <andreas@kuehntopf.org>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic.core.streamripper;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Observable;
import org.kuehntopf.streamtastic.core.Station;

/**
 * StreamRipper wrapper
 * @author akuehntopf
 */
public class StreamRipper extends Observable implements Runnable
{
    // Station
    private Station _station;
    // Name
    private String  _name;
    // URL
    private String  _url;
    // Current Track
    private String  _track;
    // Amount transferred
    private String  _transferred;
    // Destination directory
    private String  _targetDir;
    // Last Error
    private String  _error;
        
    private Process _proc;   
    private Thread  _myThread;
    private StreamRipperStatus _status;
    
    public static String STREAMRIPPER_CMD = "streamripper";
    public static String STREAMRIPPER_OPTS = "--xs2 " + // New silence det.
                                             "-m 30 ";  // Timeout 30 sec
    
    /**
     * Constructor
     * @param url URL to rip
     * @param targetDir Target directory to rip to
     */
    public 
    StreamRipper(Station s, String targetDir)
    {
        _station = s;     
        _name = s.getName();
        _url = s.getUrl();
        _targetDir = targetDir;
    }
    
    /**
     * Starts the ripping process
     * This starts a new Thread and thus
     * doesn't block.
     */
    public void 
    start()
    {
        _myThread = new Thread(this);
        _myThread.start();
    }
       
    /**
     * Stops the ripping process.
     * This waits for the Thread to come down
     * and thus blocks
     */     
    public void 
    stop()
    {
        try 
        {
            // Destroy the Process and wait for the Thread 
            // to finish
            _proc.destroy();
            _myThread.join();            
        } catch (InterruptedException ex) 
        {
            ex.printStackTrace();
        }
        
        //TODO find a way to send streamripper a SIG Term for soft shutdown, destroy is too hard.
        //on win32 platform process is killed, so  "shutting-down" message is never received
        // workaround: status is set manually
        _status = StreamRipperStatus.TERMINATED;
        setChanged();
        notifyObservers();
    }    
        
    /**
     * Parses a Statusmessage received from
     * StreamRipper
     * @param line Line to parse
     */
    private void 
    parseStatusMessage(String line)
    {   
        // Check if the Process gives us the
        // Stream title
        if (line.startsWith("stream:"))
        {
            // Skip 1 Whitespace
            _name = line.substring("stream:".length() + 1);
        } 
        else if (line.startsWith("Connecting"))
        {
            _status = StreamRipperStatus.CONNECTING;           
        }
        else if (line.startsWith("[buffering"))
        {
            _status = StreamRipperStatus.BUFFERING;
        }
        else if (line.startsWith("[re-connecting"))
       	{
        	_status = StreamRipperStatus.RECONNECTING;
    	}
        else if (line.startsWith("error"))
        {
            _status = StreamRipperStatus.ERROR;
            _error = line.substring(line.indexOf("[")+1, line.length()-1);   
        }
        else if (line.startsWith("shutting down"))
        {
            _status = StreamRipperStatus.TERMINATED;
        }
        else if (line.startsWith("[ripping"))
        {
            _status = StreamRipperStatus.RIPPING;
            int trackBegin = line.indexOf("]") + 2;
            int trackEnd = line.lastIndexOf("[") - 1;
            _track = line.substring(trackBegin, trackEnd);
            _transferred = line.substring(trackEnd + 2, line.length()-1).trim();
        }
        
        // Notify observers that we have a new
        // status
        setChanged();
        notifyObservers();
    }

    /**
     * Thread method
     * This does the actual communication across the processes
     */
    public void 
    run()
    {
        try
        {
            // Create Commandline arguments
            // and run Process
            String[] args = new String[]{STREAMRIPPER_CMD, 
                                         _url, 
                                         "-d", 
                                         _targetDir};
            _proc = Runtime.getRuntime().exec(args);
            
            // Reader for reading from the Process InputStream
            BufferedReader reader = new BufferedReader(new InputStreamReader(
                    _proc.getInputStream()));
            
            // Read lines until we should stop running or
            // the process has been finished            
            while (true)
            {                
                String line = reader.readLine();
                
                // If nothing more to read
                // cancel. This means the Process has terminated 
                // and the pipe has broken
                if (line == null)
                    break;
                
                // Else handle next line
                parseStatusMessage(line);
            }
                        
            // Close reader
            reader.close();         
            
        } catch (IOException ex)
        {
            // This happens when the process terminates and the pipe 
            // breaks. Could maybe be handled better, but for now
            // the easy solution is not to do anything.
        }
    }

    /**
     * Returns the URL this ripping is attached
     * to
     * @return URL
     */
    public String 
    getUrl()
    {
        return _url;
    }

    /**
     * Returns the current status
     * @return Status
     */
    public StreamRipperStatus 
    getStatus()
    {
        return _status;
    }

    /**
     * Returns the current track
     * @return currently ripped track
     */
    public String 
    getTrack()
    {
        return _track;
    }

    /**
     * Returns the transferred amount
     * including unit
     * @return amount transferred
     */
    public String 
    getTransferred()
    {
        return _transferred;
    }
     
    /**
     * Returns this Stream's name
     * @return name
     */
    public String 
    getName()
    {
        return _name;
    }
    
    /**
     * Returns the last Error of this Ripper
     */
    public String 
    getError() 
    {
        return _error;
    }
    
    /**
     * Returns this Ripper's Station
     */
    public Station
    getStation() 
    {
        return _station;
    }
   
    /**
     * Checks if Streamripper is actually available
     * by trying to create a process. And getting its version.
     * @return the version of the streamripper application or null
     */
    public static ProgramVersionStatus
    getVersionStatus()
    {        
        try {
            Process p = Runtime.getRuntime().exec(STREAMRIPPER_CMD + " -v");
            BufferedReader reader = new BufferedReader(new InputStreamReader(p.getInputStream()));
            String line = reader.readLine();

            // If nothing has been returned it means that this version
            // does not supply version information correctly on stdout.
            // This could mean its an older version which is not supported
            // anymore.
            if (line == null)
                return ProgramVersionStatus.NOTSUPPORTED;
            
            String version = line.split(" ")[1];

            // Further checks on the version of the executable go here.
            // Nothing to check right now, so just return OK
            return ProgramVersionStatus.OK;
            
        } catch (IOException ex) {
            return ProgramVersionStatus.NOTAVAILABLE;
        }
    }
}
