/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kühntopf <andreas@kuehntopf.org>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic.gui.views;

import java.util.ArrayList;
import java.util.List;

import javax.swing.DefaultListModel;
import javax.swing.JOptionPane;
import javax.swing.SwingWorker;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.kuehntopf.streamtastic.core.RippingManager;
import org.kuehntopf.streamtastic.core.shoutcastfeed.ShoutcastFeed;
import org.kuehntopf.streamtastic.core.shoutcastfeed.ShoutcastStation;
import org.kuehntopf.streamtastic.core.storage.Favorites;
import org.kuehntopf.streamtastic.gui.StreamtasticFrame;
import org.kuehntopf.streamtastic.gui.components.StationTableCellRenderer;
import org.kuehntopf.streamtastic.gui.models.ShoutcastTableModel;
import org.kuehntopf.streamtastic.utils.Utilities;

/**
 * Shoutcast View
 * @author  Andreas Kuehntopf
 */
public class ShoutcastView extends javax.swing.JPanel implements View
{
    private ShoutcastTableModel _tableModel;
    private DefaultListModel    _genreModel;
    private StreamtasticFrame   _parent = null;
    
    private final static int SOURCE_GENRE=0;
    private final static int SOURCE_SEARCH=1;
        
    /** 
     * Creates new form ShoutcastView 
     */
    public ShoutcastView() 
    {
        initComponents();
        
        // Setup header
        header.setCaption(Utilities.getString("SHOUT_HEADER_TEXT"));
        header.setIcon(Utilities.loadIcon("large/shoutcast.png"));        
        
        // GenreList
        _genreModel = new DefaultListModel();
        listGenres.addListSelectionListener(new ListSelectionListener() 
        {
            public void valueChanged(ListSelectionEvent arg0)
            {
                if (arg0.getValueIsAdjusting() == true) return;
                refreshSelectedGenre();                
                updateButtons();
            }
        });
        
        // Setup Station table
        _tableModel = new ShoutcastTableModel();        
        tblStations.setModel(_tableModel);
        tblStations.setDefaultRenderer(Object.class, new StationTableCellRenderer());
        tblStations.setAutoCreateRowSorter(true);
        tblStations.getSelectionModel().addListSelectionListener(new ListSelectionListener() 
        {
            public void valueChanged(ListSelectionEvent arg0) 
            {
                updateButtons();
            }
        });
        
        updateButtons();
    }
    
            
    public ShoutcastView(StreamtasticFrame parent)
    {        
        this();
        _parent = parent;
    }
    
                
    private void
    updateButtons()
    {
        // Set Refresh button
        if (getSelectedGenre() == null)
        {
            btnRefresh.setVisible(false);
        } else
        {
            btnRefresh.setVisible(true);            
        }
        
        // Set Other buttons
        ShoutcastStation s = getSelectedStation();
        if (s == null)
        {
            btnAddToFavs.setVisible(false);
            btnRecord.setVisible(false);
        } else 
        {
            btnAddToFavs.setVisible(true);
            if (!RippingManager.getInstance().isRipping(s))
                btnRecord.setVisible(true);
            else
                btnRecord.setVisible(false);
        }
    }
    
    private String
    getSelectedGenre()
    {
        int selPos = listGenres.getSelectedIndex();
        if (selPos == -1) return null;
        
        return (String)_genreModel.get(selPos);
    }
    
    private ShoutcastStation
    getSelectedStation()
    {
        int selPos = tblStations.getSelectedRow();
        if (selPos == -1) return null;
        
        return (ShoutcastStation) tblStations.getValueAt(selPos, 0);
    }
    
    /**
     * Gets called when the View is activated
     */
    public void activate()
    {                        
        // Don't reload if there already are some genres    
        if (_genreModel.getSize() > 0)
            return;        

        SwingWorker<List<String>, Void> worker = new SwingWorker<List<String>, Void>() 
        {
            @Override
            protected List<String> doInBackground() throws Exception 
            {                
                return ShoutcastFeed.getGenres();
            }

            @Override
            protected void done() 
            {
                _parent.setBusy(false);
                try
                {
                    List<String> genres = get();
                    for (String genre: genres) _genreModel.addElement(genre);
                    listGenres.setModel(_genreModel);                    
                } catch (Exception e)
                {
                    if (JOptionPane.YES_OPTION==JOptionPane.showConfirmDialog(
                            _parent,Utilities.getString("SHOUT_GENRELIST_ERROR_MESSAGE"),
                            Utilities.getString("SHOUT_GENRELIST_ERROR_TITLE"),
                            JOptionPane.YES_NO_OPTION,
                            JOptionPane.ERROR_MESSAGE)
                        ) activate();
                }
            }
        };
        
        _parent.setBusy(true);
        worker.execute();    
    }
    
    private void
    recordSelected()
    {
        ShoutcastStation s = getSelectedStation();
        if (RippingManager.getInstance().isRipping(s))
        {
            Utilities.showLocalizedError("STREAMRIPPER_ALREADY_RECORDING");
            return;
        }
        
        // Start the Rip
        RippingManager.getInstance().createRip(s);   
        
        // Redraw the Table to reflect it
        tblStations.repaint();
        
        // Update Buttons
        updateButtons();
    }
    
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jSplitPane1 = new javax.swing.JSplitPane();
        jScrollPane1 = new javax.swing.JScrollPane();
        listGenres = new javax.swing.JList();
        jScrollPane2 = new javax.swing.JScrollPane();
        tblStations = new javax.swing.JTable();
        header = new org.kuehntopf.streamtastic.gui.components.GlossyPanel();
        toolbar = new javax.swing.JToolBar();
        btnRecord = new javax.swing.JButton();
        btnAddToFavs = new javax.swing.JButton();
        btnRefresh = new javax.swing.JButton();
        lblSearch = new javax.swing.JLabel();
        cbBitrate = new javax.swing.JComboBox();
        jLabel1 = new javax.swing.JLabel();
        tfSearch = new javax.swing.JTextField();

        jSplitPane1.setDividerLocation(150);

        listGenres.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        jScrollPane1.setViewportView(listGenres);

        jSplitPane1.setLeftComponent(jScrollPane1);

        tblStations.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                tblStationsMouseClicked(evt);
            }
        });
        jScrollPane2.setViewportView(tblStations);

        jSplitPane1.setRightComponent(jScrollPane2);

        toolbar.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(126, 126, 126)));
        toolbar.setFloatable(false);
        toolbar.setRollover(true);

        btnRecord.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/kuehntopf/streamtastic/resources/icons/small/record.png"))); // NOI18N
        btnRecord.setToolTipText(Utilities.getString("RECORD")); // NOI18N
        btnRecord.setFocusable(false);
        btnRecord.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        btnRecord.setMaximumSize(new java.awt.Dimension(48, 48));
        btnRecord.setMinimumSize(new java.awt.Dimension(48, 48));
        btnRecord.setPreferredSize(new java.awt.Dimension(32, 32));
        btnRecord.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        btnRecord.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnRecordActionPerformed(evt);
            }
        });
        toolbar.add(btnRecord);

        btnAddToFavs.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/kuehntopf/streamtastic/resources/icons/small/favorites.png"))); // NOI18N
        btnAddToFavs.setToolTipText(Utilities.getString("SHOUT_ADD_TO_FAVS")); // NOI18N
        btnAddToFavs.setFocusable(false);
        btnAddToFavs.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        btnAddToFavs.setMaximumSize(new java.awt.Dimension(48, 48));
        btnAddToFavs.setMinimumSize(new java.awt.Dimension(48, 48));
        btnAddToFavs.setPreferredSize(new java.awt.Dimension(32, 32));
        btnAddToFavs.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        btnAddToFavs.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnAddToFavsActionPerformed(evt);
            }
        });
        toolbar.add(btnAddToFavs);

        btnRefresh.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/kuehntopf/streamtastic/resources/icons/small/refresh.png"))); // NOI18N
        btnRefresh.setToolTipText(Utilities.getString("REFRESH")); // NOI18N
        btnRefresh.setFocusable(false);
        btnRefresh.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        btnRefresh.setMaximumSize(new java.awt.Dimension(48, 48));
        btnRefresh.setMinimumSize(new java.awt.Dimension(48, 48));
        btnRefresh.setPreferredSize(new java.awt.Dimension(32, 32));
        btnRefresh.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        btnRefresh.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnRefreshActionPerformed(evt);
            }
        });
        toolbar.add(btnRefresh);

        lblSearch.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("org/kuehntopf/streamtastic/resources/i18n/streamtastic"); // NOI18N
        lblSearch.setText(bundle.getString("SHOUT_SEARCH_LBL")); // NOI18N
        lblSearch.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 2, 0, 5));

        cbBitrate.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "all", "68", "96", "128", "160", "192", "256", "320" }));
        cbBitrate.setBorder(javax.swing.BorderFactory.createLineBorder(java.awt.SystemColor.inactiveCaptionBorder));
        cbBitrate.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbBitrateActionPerformed(evt);
            }
        });

        jLabel1.setText(bundle.getString("SHOUT_SEARCH_BR")); // NOI18N

        tfSearch.setToolTipText(Utilities.getString("SHOUT_SEARCH_TT")); // NOI18N
        tfSearch.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                tfSearchActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(layout.createSequentialGroup()
                        .addGap(12, 12, 12)
                        .addComponent(toolbar, javax.swing.GroupLayout.DEFAULT_SIZE, 626, Short.MAX_VALUE))
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(jSplitPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 626, Short.MAX_VALUE))
                    .addComponent(header, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 638, Short.MAX_VALUE)
                    .addGroup(layout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(lblSearch)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(tfSearch, javax.swing.GroupLayout.DEFAULT_SIZE, 371, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabel1)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(cbBitrate, javax.swing.GroupLayout.PREFERRED_SIZE, 116, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(header, javax.swing.GroupLayout.PREFERRED_SIZE, 50, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(cbBitrate, javax.swing.GroupLayout.PREFERRED_SIZE, 23, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel1)
                    .addComponent(lblSearch)
                    .addComponent(tfSearch, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jSplitPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 253, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(toolbar, javax.swing.GroupLayout.PREFERRED_SIZE, 29, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
    }// </editor-fold>//GEN-END:initComponents

    private void 
    refreshSelectedGenre()
    {
        if (getSelectedGenre()==null) 
            return;
        
        refreshStationTable(getSelectedGenre(),SOURCE_GENRE);
        tfSearch.setText("");
    }
    
    private void
    refreshStationTable(final String query, final int source)
    {              
        // Clear model
        _tableModel.clear();
        if (source==SOURCE_SEARCH && query.length()<3)
        {
            JOptionPane.showMessageDialog(_parent, 
                    Utilities.getString("SHOUT_SEARCH_ERROR_MESSAGE"),
                    Utilities.getString("SHOUT_STATIONLIST_ERROR_TITLE"), 
                    JOptionPane.ERROR_MESSAGE);
            return;
        }
                
        SwingWorker<ArrayList<ShoutcastStation>, Void> worker = new SwingWorker<ArrayList<ShoutcastStation>, Void>() 
        {
            @Override
            protected ArrayList<ShoutcastStation> doInBackground() throws Exception 
            {
                String bitrate=(cbBitrate.getSelectedIndex()!=0)?(String) cbBitrate.getSelectedItem():null;
                
                switch (source)
                {
                    case SOURCE_GENRE: 
                        // retrieve new stations
                        return ShoutcastFeed.getStations(query,bitrate);
                    case SOURCE_SEARCH:
                        return ShoutcastFeed.getSearchResult(query,bitrate);
                    default:
                      return null;
                }
            }

            @Override
            protected void done() 
            {   
                _parent.setBusy(false);
                try
                {
                    ArrayList<ShoutcastStation> stations = get();
                    _tableModel.setStations(stations);
                } catch (Exception e)
                {
                    if (JOptionPane.YES_OPTION==JOptionPane.showConfirmDialog(
                            _parent, Utilities.getString("SHOUT_STATIONLIST_ERROR_MESSAGE"), Utilities.getString("SHOUT_STATIONLIST_ERROR_TITLE"), 
                            JOptionPane.YES_NO_OPTION, JOptionPane.ERROR_MESSAGE)
                        )refreshStationTable(query,source);
                }
            }
        };

        _parent.setBusy(true);
        worker.execute();                
    }
    
    private void tblStationsMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_tblStationsMouseClicked
        if (evt.getClickCount() == 2)
        {
            recordSelected();
        }
    }//GEN-LAST:event_tblStationsMouseClicked

private void btnRecordActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnRecordActionPerformed
    ShoutcastStation current = getSelectedStation();
    RippingManager.getInstance().createRip(current);
}//GEN-LAST:event_btnRecordActionPerformed

private void btnRefreshActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnRefreshActionPerformed
    refreshSelectedGenre();
}//GEN-LAST:event_btnRefreshActionPerformed

private void btnAddToFavsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnAddToFavsActionPerformed
    ShoutcastStation current = getSelectedStation();
    Favorites.getInstance().addFavorite(current.getUrl(), current.getName());
}//GEN-LAST:event_btnAddToFavsActionPerformed

private void cbBitrateActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbBitrateActionPerformed
    if (getSelectedGenre()!=null) refreshSelectedGenre();
    else if (tfSearch.getText().length()>2) refreshStationTable(tfSearch.getText(), SOURCE_SEARCH);
}//GEN-LAST:event_cbBitrateActionPerformed

private void tfSearchActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_tfSearchActionPerformed
    _tableModel.clear();
    refreshStationTable(tfSearch.getText(), SOURCE_SEARCH);
}//GEN-LAST:event_tfSearchActionPerformed
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnAddToFavs;
    private javax.swing.JButton btnRecord;
    private javax.swing.JButton btnRefresh;
    private javax.swing.JComboBox cbBitrate;
    private org.kuehntopf.streamtastic.gui.components.GlossyPanel header;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JLabel lblSearch;
    private javax.swing.JList listGenres;
    private javax.swing.JTable tblStations;
    private javax.swing.JTextField tfSearch;
    private javax.swing.JToolBar toolbar;
    // End of variables declaration//GEN-END:variables
    
}
